@@ -1,7 +1,7 @@
 
 //OpenSCADA module DAQ.BFN file: mod_BFN.cpp
 /***************************************************************************
- *   Copyright (C) 2010-2018 by Roman Savochenko, <rom_as@oscada.org>      *
+ *   Copyright (C) 2010-2019 by Roman Savochenko, <rom_as@oscada.org>      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -36,7 +36,7 @@
 #define MOD_NAME	_("BFN module")
 #define MOD_TYPE	SDAQ_ID
 #define VER_TYPE	SDAQ_VER
-#define MOD_VER		"0.6.13"
+#define MOD_VER		"0.6.18"
 #define AUTHORS		_("Roman Savochenko")
 #define DESCRIPTION	_("Support Big Farm Net (BFN) modules for Viper CT/BAS and other from \"Big Dutchman\" (http://www.bigdutchman.com).")
 #define LICENSE		"GPL2"
@@ -117,7 +117,7 @@ void TTpContr::setSymbDB( const string &idb )
 
 string TTpContr::getSymbolCode(const string &id)
 {
-    MtxAlloc res(dataRes(), true);
+    MtxAlloc res(mSymbRes, true);
     map<unsigned,string>::iterator is = mSymbCode.find(s2i(id));
     if(is == mSymbCode.end()) return TSYS::strMess(_("Code %s"),id.c_str());
     return is->second;
@@ -125,7 +125,7 @@ string TTpContr::getSymbolCode(const string &id)
 
 TTpContr::AlrmSymb TTpContr::getSymbolAlarm(const string &id)
 {
-    MtxAlloc res(dataRes(), true);
+    MtxAlloc res(mSymbRes, true);
     map<unsigned,AlrmSymb>::iterator is = mSymbAlrm.find(s2i(id));
     if(is == mSymbAlrm.end()) return AlrmSymb();
     return is->second;
@@ -137,7 +137,7 @@ void TTpContr::load_( )
     string wtbl = MOD_ID"_SymbCode";
     string wdb  = symbDB();
     TConfig c_el(&symbCode_el);
-    MtxAlloc res(dataRes(), true);
+    MtxAlloc res(mSymbRes, true);
     mSymbCode.clear();
     for(int fld_cnt = 0; SYS->db().at().dataSeek(wdb+"."+wtbl,nodePath()+wtbl,fld_cnt,c_el); fld_cnt++)
 	mSymbCode[c_el.cfg("ID").getI()] = c_el.cfg("TEXT").getS();
@@ -156,7 +156,7 @@ void TTpContr::save_( )
     string wtbl = MOD_ID"_SymbCode";
     string wdb  = symbDB();
     TConfig c_el(&symbCode_el);
-    MtxAlloc res(dataRes(), true);
+    MtxAlloc res(mSymbRes, true);
     for(map<unsigned,string>::iterator is = mSymbCode.begin(); is != mSymbCode.end(); is++) {
 	c_el.cfg("ID").setI(is->first);
 	c_el.cfg("TEXT").setS(is->second);
@@ -216,7 +216,7 @@ void TTpContr::cntrCmdProc( XMLNode *opt )
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR))	setSymbDB(opt->text());
     }
     else if(a_path == "/symbs/codes") {
-	MtxAlloc res(dataRes(), true);
+	MtxAlloc res(mSymbRes, true);
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD)) {
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/symbs/codes/id","");
 	    XMLNode *n_text	= ctrMkNode("list",opt,-1,"/symbs/codes/text","");
@@ -243,7 +243,7 @@ void TTpContr::cntrCmdProc( XMLNode *opt )
 	modif();
     }
     else if(a_path == "/symbs/alrms") {
-	MtxAlloc res(dataRes(), true);
+	MtxAlloc res(mSymbRes, true);
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD)) {
 	    XMLNode *n_id	= ctrMkNode("list",opt,-1,"/symbs/alrms/id","");
 	    XMLNode *n_code	= ctrMkNode("list",opt,-1,"/symbs/alrms/code","");
@@ -280,7 +280,7 @@ void TTpContr::cntrCmdProc( XMLNode *opt )
 //*************************************************
 TMdContr::TMdContr(string name_c, const string &daq_db, ::TElem *cfgelem) : ::TController(name_c,daq_db,cfgelem),
 	mPrior(cfg("PRIOR").getId()), mSync(cfg("SYNCPER").getRd()),
-	prc_st(false), acq_st(false), endrun_req(false), alSt(-1), tm_gath(0)
+	mPer(1e9), prc_st(false), acq_st(false), endrun_req(false), alSt(-1), tm_gath(0)
 {
     //cfg("PRM_BD").setS("TmplPrm_"+name_c);
 }
@@ -377,9 +377,6 @@ void TMdContr::enable_( )
 
 void TMdContr::start_( )
 {
-    //Schedule process
-    mPer = TSYS::strSepParse(cron(),1,' ').empty() ? vmax(0,(int64_t)(1e9*s2r(cron()))) : 0;
-
     //Start the gathering data task
     if(!prc_st) SYS->taskCreate(nodePath('.',true), mPrior, TMdContr::Task, this);
 }
@@ -389,7 +386,7 @@ void TMdContr::stop_( )
     //Stop the request and calc data task
     if(prc_st) SYS->taskDestroy(nodePath('.',true), &endrun_req);
 
-    alarmSet(TSYS::strMess(_("DAQ.%s.%s: connect to the data source: %s."),owner().modId().c_str(),id().c_str(),_("STOP")), TMess::Info);
+    alarmSet(TSYS::strMess(_("Connection to the data source: %s."),_("STOP")), TMess::Info);
     alSt = -1;
 
     //Clear errors and set EVal
@@ -524,7 +521,7 @@ void *TMdContr::Task( void *icntr )
 
 	    //Update controller's data
 	    ResAlloc res(cntr.en_res,false);
-	    for(unsigned i_p=0; i_p < cntr.p_hd.size() && !cntr.redntUse() && !cntr.endrun_req; i_p++) {
+	    for(unsigned i_p = 0; i_p < cntr.p_hd.size() && !cntr.redntUse() && !cntr.endrun_req; i_p++) {
 		//Get current data
 		XMLNode reqCodeData("GetCodeData");
 		reqCodeData.childAdd("lHouseComputerId")->setText(TSYS::strParse(cntr.p_hd[i_p].at().id(),1,"_hc"));
@@ -585,13 +582,11 @@ void *TMdContr::Task( void *icntr )
 	//Generic alarm generate
 	if(tErr.size() && cntr.alSt <= 0) {
 	    cntr.alSt = 1;
-	    cntr.alarmSet(TSYS::strMess(_("DAQ.%s.%s: connect to the data source: %s."),cntr.owner().modId().c_str(),cntr.id().c_str(),
-						TRegExp(":","g").replace(tErr,"=").c_str()));
+	    cntr.alarmSet(TSYS::strMess(_("Connection to the data source: %s."),TRegExp(":","g").replace(tErr,"=").c_str()));
 	}
 	else if(!tErr.size() && cntr.alSt != 0) {
 	    cntr.alSt = 0;
-	    cntr.alarmSet(TSYS::strMess(_("DAQ.%s.%s: connect to the data source: %s."),cntr.owner().modId().c_str(),cntr.id().c_str(),_("OK")),
-			    TMess::Info);
+	    cntr.alarmSet(TSYS::strMess(_("Connection to the data source: %s."),_("OK")), TMess::Info);
 	}
 	cntr.acq_err.setVal(tErr);
 
@@ -654,6 +649,16 @@ void TMdContr::cntrCmdProc( XMLNode *opt )
     else TController::cntrCmdProc(opt);
 }
 
+bool TMdContr::cfgChange( TCfg &co, const TVariant &pc )
+{
+    TController::cfgChange(co, pc);
+
+    if(co.fld().name() == "SCHEDULE")
+	mPer = TSYS::strSepParse(cron(),1,' ').empty() ? vmax(0,(int64_t)(1e9*s2r(cron()))) : 0;
+
+    return true;
+}
+
 //*************************************************
 //* TMdPrm                                        *
 //*************************************************
