@@ -1,7 +1,7 @@
 
 //OpenSCADA module UI.Vision file: vis_devel_dlgs.cpp
 /***************************************************************************
- *   Copyright (C) 2007-2018 by Roman Savochenko, <rom_as@oscada.org>      *
+ *   Copyright (C) 2007-2019 by Roman Savochenko, <rom_as@oscada.org>      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -27,6 +27,7 @@
 #include <QGridLayout>
 #include <QCheckBox>
 #include <QComboBox>
+#include <QTextEdit>
 #include <QDialogButtonBox>
 #include <QBuffer>
 #include <QTableWidget>
@@ -37,11 +38,8 @@
 #include <QApplication>
 #include <QDesktopWidget>
 
-#include <tsys.h>
-
-#include "vis_devel.h"
-#include "tvision.h"
 #include "vis_widgs.h"
+#include "vis_devel.h"
 #include "vis_devel_dlgs.h"
 
 using namespace VISION;
@@ -92,26 +90,35 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     obj_ico->setIconSize(QSize(icoSize(5),icoSize(5)));
     obj_ico->setAutoDefault(false);
     connect(obj_ico, SIGNAL(released()), this, SLOT(selectIco()));
-    glay->addWidget(obj_ico, 0, 0, 4, 1);
+    glay->addWidget(obj_ico, 0, 0, 3, 1);
+
+    lab = new QLabel(_("Status:"), tab_w);
+    glay->addWidget(lab, 0, 1);
+    obj_st = new QLabel(tab_w);
+    obj_st->setTextInteractionFlags(Qt::TextSelectableByMouse);
+    obj_st->setObjectName("/obj/st/status");
+    obj_st->setFrameStyle(QFrame::Panel | QFrame::Sunken);
+    obj_st->setWordWrap(true);
+    glay->addWidget(obj_st, 0, 2, 1, 4);
 
     lab = new QLabel(_("Enabled:"), tab_w);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
-    glay->addWidget(lab, 0, 1);
+    glay->addWidget(lab, 1, 1);
     obj_enable = new QCheckBox(tab_w);
     obj_enable->setObjectName("/obj/st/en");
     connect(obj_enable, SIGNAL(stateChanged(int)), this, SLOT(isModify()));
-    glay->addWidget(obj_enable, 0, 2);
+    glay->addWidget(obj_enable, 1, 2);
 
     lab = new QLabel(_("Container DB:"), tab_w);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
-    glay->addWidget(lab, 1, 1);
+    glay->addWidget(lab, 2, 1);
     obj_db = new LineEdit(tab_w, LineEdit::Combo);
     obj_db->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed));
     obj_db->setObjectName("/obj/st/db");
     connect(obj_db, SIGNAL(apply()), this, SLOT(isModify()));
-    glay->addWidget(obj_db, 1, 2);
+    glay->addWidget(obj_db, 2, 2);
 
-    lab = new QLabel(_("Used:"), tab_w);
+    /*lab = new QLabel(_("Used:"), tab_w);
     glay->addWidget(lab, 2, 1);
     obj_used = new QLabel(tab_w);
     obj_used->setTextInteractionFlags(Qt::TextSelectableByMouse);
@@ -125,7 +132,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     obj_tmstmp->setTextInteractionFlags(Qt::TextSelectableByMouse);
     obj_tmstmp->setObjectName("/obj/st/timestamp");
     obj_tmstmp->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    glay->addWidget(obj_tmstmp, 3, 2);
+    glay->addWidget(obj_tmstmp, 3, 2);*/
 
     grp->setLayout(glay);
     dlg_lay->addWidget(grp, 0, 0);
@@ -144,7 +151,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     glay->addWidget(obj_id, 0, 1, 1, 3);
 
     glay->addWidget(new QLabel(_("Name:"),tab_w), 1, 0);
-    obj_name = new LineEdit(this);
+    obj_name = new LineEdit(this, LineEdit::Text, false, false);
     obj_name->setObjectName("/obj/cfg/name");
     connect(obj_name, SIGNAL(apply()), this, SLOT(isModify()));
     glay->addWidget(obj_name, 1, 1, 1, 3);
@@ -246,7 +253,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     dlg_lay->addWidget(buttStlDel,0,3);
 
     dlg_lay->addWidget(new QLabel(_("Name:"),tab_w), 1, 0);
-    stl_name = new LineEdit(tab_w);
+    stl_name = new LineEdit(tab_w, LineEdit::Text, false, false);
     stl_name->setObjectName("/style/name");
     connect(stl_name, SIGNAL(apply()), this, SLOT(isModify()));
     dlg_lay->addWidget(stl_name, 1, 1, 1, 2);
@@ -272,7 +279,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     lab = new QLabel(_("Time:"), tab_w);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
     dlg_lay->addWidget(lab, 0, 0);
-    messTime = new LineEdit(tab_w, LineEdit::DateTime);
+    messTime = new LineEdit(tab_w, LineEdit::DateTime, false, false);
     messTime->setCfg("dd-MM-yyyy hh:mm:ss");
     messTime->setObjectName("/mess/tm");
     connect(messTime, SIGNAL(apply()), this, SLOT(isModify()));
@@ -287,7 +294,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     lab = new QLabel(_("Size:"), tab_w);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
     dlg_lay->addWidget(lab, 1, 0);
-    messSize = new LineEdit(tab_w, LineEdit::Integer);
+    messSize = new LineEdit(tab_w, LineEdit::Integer, false, false);
     messSize->setCfg("1:1000:1:: s");
     messSize->setObjectName("/mess/size");
     connect(messSize, SIGNAL(apply()), this, SLOT(isModify()));
@@ -358,6 +365,12 @@ void LibProjProp::showDlg( const string &iit, bool reload )
     wdg_tabs->setTabEnabled(0,gnd);
     if(gnd) {
 	wdg_tabs->setTabText(0,gnd->attr("dscr").c_str());
+	// Status
+	gnd = TCntrNode::ctrId(root, obj_st->objectName().toStdString(), true);
+	if(gnd) {
+	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_st->objectName().toStdString(),TSYS::PathEl));
+	    if(!owner()->cntrIfCmd(req)) obj_st->setText(req.text().c_str());
+	}else obj_st->setText("");
 	// Enable stat
 	gnd = TCntrNode::ctrId(root,obj_enable->objectName().toStdString(),true);
 	obj_enable->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
@@ -380,7 +393,7 @@ void LibProjProp::showDlg( const string &iit, bool reload )
 	    }
 	}
 	// Used
-	gnd = TCntrNode::ctrId(root, obj_used->objectName().toStdString(), true);
+	/*gnd = TCntrNode::ctrId(root, obj_used->objectName().toStdString(), true);
 	if(gnd) {
 	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_used->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_used->setText(req.text().c_str());
@@ -391,7 +404,7 @@ void LibProjProp::showDlg( const string &iit, bool reload )
 	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_tmstmp->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_tmstmp->setText(atm2s(s2i(req.text())).c_str());
 	    else obj_tmstmp->setText(_("Not set"));
-	}else obj_tmstmp->setText("");
+	}else obj_tmstmp->setText("");*/
 	// User
 	gnd = TCntrNode::ctrId(root,obj_user->objectName().toStdString(),true);
 	obj_user->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
@@ -478,6 +491,11 @@ void LibProjProp::showDlg( const string &iit, bool reload )
 	gnd = TCntrNode::ctrId(root,obj_descr->objectName().toStdString(),true);
 	obj_descr->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
 	if(gnd) {
+	    if(s2i(gnd->attr("cols"))) {
+		obj_descr->workWdg()->setWordWrapMode(QTextOption::WordWrap);
+		obj_descr->workWdg()->setLineWrapMode(QTextEdit::FixedColumnWidth);
+		obj_descr->workWdg()->setLineWrapColumnOrWidth(s2i(gnd->attr("cols")));
+	    }
 	    req.clear()->setAttr("path", ed_it+"/"+TSYS::strEncode(obj_descr->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_descr->setText(req.text().c_str());
 	}
@@ -745,19 +763,25 @@ void LibProjProp::isModify( QObject *snd )
     is_modif = true;
 }
 
+void LibProjProp::keyPressEvent( QKeyEvent *ce )
+{
+    if(ce->key() == Qt::Key_Escape) ce->ignore();
+    else QDialog::keyPressEvent(ce);
+}
+
 void LibProjProp::closeEvent( QCloseEvent *ce )
 {
     //Check for not apply LineEdit and TextEdit widgets
     QList<LineEdit*> lnEdWs = findChildren<LineEdit*>();
     QList<TextEdit*> txtEdWs = findChildren<TextEdit*>();
     bool notApplyPresent = false;
-    for(int i_it = 0; !notApplyPresent && i_it < lnEdWs.size(); ++i_it) notApplyPresent = lnEdWs[i_it]->isEdited();
-    for(int i_it = 0; !notApplyPresent && i_it < txtEdWs.size(); ++i_it) notApplyPresent = txtEdWs[i_it]->isEdited();
+    for(int iIt = 0; !notApplyPresent && iIt < lnEdWs.size(); ++iIt) notApplyPresent = lnEdWs[iIt]->isEdited();
+    for(int iIt = 0; !notApplyPresent && iIt < txtEdWs.size(); ++iIt) notApplyPresent = txtEdWs[iIt]->isEdited();
     if(notApplyPresent && QMessageBox::information(this,_("Saving the changes"),_("Some changes were made!\nSave the changes to the DB before the closing?"),
 	    QMessageBox::Apply|QMessageBox::Cancel,QMessageBox::Apply) == QMessageBox::Apply)
     {
-	for(int i_it = 0; i_it < lnEdWs.size(); ++i_it) if(lnEdWs[i_it]->isEdited()) isModify(lnEdWs[i_it]);
-	for(int i_it = 0; i_it < txtEdWs.size(); ++i_it) if(txtEdWs[i_it]->isEdited()) isModify(txtEdWs[i_it]);
+	for(int iIt = 0; iIt < lnEdWs.size(); ++iIt) if(lnEdWs[iIt]->isEdited()) isModify(lnEdWs[iIt]);
+	for(int iIt = 0; iIt < txtEdWs.size(); ++iIt) if(txtEdWs[iIt]->isEdited()) isModify(txtEdWs[iIt]);
     }
 
     if(is_modif) emit apply(ed_it);
@@ -869,7 +893,7 @@ void LibProjProp::unloadMimeData( )
 
 void LibProjProp::mimeDataChange( int row, int column )
 {
-    if( show_init ) return;
+    if(show_init) return;
 
     XMLNode req("set");
     req.setAttr("path",ed_it+"/"+TSYS::strEncode("/mime/mime",TSYS::PathEl))->
@@ -930,7 +954,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
 
     //Add tab 'Widget'
     QScrollArea *scrl = new QScrollArea();
-    wdg_tabs->addTab(scrl,_("Widget"));
+    wdg_tabs->addTab(scrl, _("Widget"));
     QWidget *tab_w = new QWidget(scrl);
     scrl->setWidget(tab_w);
     scrl->setWidgetResizable(true);
@@ -952,24 +976,24 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     obj_ico->setIconSize(QSize(icoSize(5),icoSize(5)));
     obj_ico->setAutoDefault(false);
     connect(obj_ico, SIGNAL(released()), this, SLOT(selectIco()));
-    glay->addWidget(obj_ico,0,0,5,1);
+    glay->addWidget(obj_ico,0,0,4,1);
+
+    lab = new QLabel(_("Status:"), tab_w);
+    glay->addWidget(lab, 0, 1);
+    obj_st = new QLabel(tab_w);
+    obj_st->setTextInteractionFlags(Qt::TextSelectableByMouse);
+    obj_st->setObjectName("/wdg/st/status");
+    obj_st->setFrameStyle(QFrame::Panel | QFrame::Sunken);
+    obj_st->setWordWrap(true);
+    glay->addWidget(obj_st, 0, 2, 1, 4);
 
     lab = new QLabel(_("Enabled:"),tab_w);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred));
-    glay->addWidget(lab,0,1);
+    glay->addWidget(lab, 1, 1);
     obj_enable = new QCheckBox(tab_w);
     obj_enable->setObjectName("/wdg/st/en");
     connect(obj_enable, SIGNAL(stateChanged(int)), this, SLOT(isModify()));
-    glay->addWidget(obj_enable,0,2,1,4);
-
-    lab = new QLabel(_("Parent widget:"), tab_w);
-    lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred));
-    glay->addWidget(lab, 1, 1);
-    obj_parent = new QComboBox(tab_w);
-    obj_parent->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
-    obj_parent->setObjectName("/wdg/st/parent");
-    connect(obj_parent, SIGNAL(activated(int)), this, SLOT(isModify()));
-    glay->addWidget(obj_parent, 1, 2, 1, 4);
+    glay->addWidget(obj_enable, 1, 2, 1, 4);
 
     //  Specific parameter: page type
     lab = new QLabel(_("Page type:"), tab_w);
@@ -980,7 +1004,16 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     connect(pg_tp, SIGNAL(currentIndexChanged(int)), this, SLOT(isModify()));
     glay->addWidget(pg_tp, 2, 2);
 
-    lab = new QLabel(_("Used:"), tab_w);
+    lab = new QLabel(_("Parent widget:"), tab_w);
+    lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred));
+    glay->addWidget(lab, 3, 1);
+    obj_parent = new QComboBox(tab_w);
+    obj_parent->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
+    obj_parent->setObjectName("/wdg/st/parent");
+    connect(obj_parent, SIGNAL(activated(int)), this, SLOT(isModify()));
+    glay->addWidget(obj_parent, 3, 2, 1, 4);
+
+    /*lab = new QLabel(_("Used:"), tab_w);
     glay->addWidget(lab, 3, 1);
     obj_used = new QLabel(tab_w);
     obj_used->setTextInteractionFlags(Qt::TextSelectableByMouse);
@@ -994,10 +1027,10 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     obj_tmstmp->setTextInteractionFlags(Qt::TextSelectableByMouse);
     obj_tmstmp->setObjectName("/wdg/st/timestamp");
     obj_tmstmp->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    glay->addWidget(obj_tmstmp, 4, 2, 1, 4);
+    glay->addWidget(obj_tmstmp, 4, 2, 1, 4);*/
 
     grp->setLayout(glay);
-    dlg_lay->addWidget(grp,0,0);
+    dlg_lay->addWidget(grp, 0, 0);
 
     //  Configuration parameters
     grp = new QGroupBox(_("Configuration"),tab_w);
@@ -1005,41 +1038,41 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     glay->setMargin(4);
     glay->setSpacing(6);
 
-    glay->addWidget(new QLabel(_("Identifier:"),tab_w),0,0);
+    glay->addWidget(new QLabel(_("Identifier:"),tab_w), 0, 0);
     obj_id = new QLabel(tab_w);
     obj_id->setTextInteractionFlags(Qt::TextSelectableByMouse);
     obj_id->setObjectName("/wdg/cfg/id");
     obj_id->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    glay->addWidget(obj_id,0,1,1,3);
+    glay->addWidget(obj_id, 0, 1, 1, 3);
 
-    glay->addWidget(new QLabel(_("Root:"),tab_w),1,0);
+    glay->addWidget(new QLabel(_("Root:"),tab_w), 1, 0);
     obj_root = new QLabel(tab_w);
     obj_root->setTextInteractionFlags(Qt::TextSelectableByMouse);
     obj_root->setObjectName("/wdg/cfg/root");
     obj_root->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    glay->addWidget(obj_root,1,1,1,3);
+    glay->addWidget(obj_root, 1, 1, 1, 3);
 
-    glay->addWidget(new QLabel(_("Path:"),tab_w),2,0);
+    glay->addWidget(new QLabel(_("Path:"),tab_w), 2, 0);
     obj_path = new QLabel(tab_w);
     obj_path->setTextInteractionFlags(Qt::TextSelectableByMouse);
     obj_path->setObjectName("/wdg/cfg/path");
     obj_path->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    glay->addWidget(obj_path,2,1,1,3);
+    glay->addWidget(obj_path, 2, 1, 1, 3);
 
-    glay->addWidget(new QLabel(_("Name:"),tab_w),3,0);
-    obj_name = new LineEdit(tab_w);
+    glay->addWidget(new QLabel(_("Name:"),tab_w), 3, 0);
+    obj_name = new LineEdit(tab_w, LineEdit::Text, false, false);
     obj_name->setObjectName("/wdg/cfg/name");
     connect(obj_name, SIGNAL(apply()), this, SLOT(isModify()));
-    glay->addWidget(obj_name,3,1,1,3);
+    glay->addWidget(obj_name, 3, 1, 1, 3);
 
-    glay->addWidget(new QLabel(_("Description:"),tab_w),4,0);
+    glay->addWidget(new QLabel(_("Description:"),tab_w), 4, 0);
     obj_descr = new TextEdit(tab_w);
     obj_descr->setObjectName("/wdg/cfg/descr");
     connect(obj_descr, SIGNAL(apply()), this, SLOT(isModify()));
-    glay->addWidget(obj_descr,5,0,1,4);
+    glay->addWidget(obj_descr, 5, 0, 1, 4);
 
     grp->setLayout(glay);
-    dlg_lay->addWidget(grp,1,0);
+    dlg_lay->addWidget(grp, 1, 0);
 
     //Add tab 'Attributes'
     wdg_tabs->addTab(new QWidget,_("Attributes"));
@@ -1052,7 +1085,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     // Add attributes view widget
     obj_attr = new InspAttr(tab_w,owner());
     connect(obj_attr, SIGNAL(modified(const string&)), this, SIGNAL(apply(const string&)));
-    dlg_lay->addWidget(obj_attr,0,0);
+    dlg_lay->addWidget(obj_attr, 0, 0);
 
     //Add tab 'Attribute cofiguration'
     proc_split = new QSplitter();
@@ -1078,14 +1111,14 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     connect(obj_attr_cfg, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(changeAttr(QTreeWidgetItem*,int)));
     obj_attr_cfg->setHeaderLabels(QStringList() << _("Identifier") << _("Name") << _("Data type") << _("Work area") << _("Processing")
 						<< _("Configuration") << _("Configuration template"));
-    glay->addWidget(obj_attr_cfg,0,0,1,2);
+    glay->addWidget(obj_attr_cfg, 0, 0, 1, 2);
 
     buttAttrAdd = new QPushButton(_("Add attribute"),attr_cf_fr); 
     connect(buttAttrAdd, SIGNAL(clicked()), this, SLOT(addAttr()));
-    glay->addWidget(buttAttrAdd,1,0);
+    glay->addWidget(buttAttrAdd, 1, 0);
     buttAttrDel = new QPushButton(_("Delete attribute"),attr_cf_fr);
     connect(buttAttrDel, SIGNAL(clicked()), this, SLOT(delAttr()));
-    glay->addWidget(buttAttrDel,1,1);
+    glay->addWidget(buttAttrDel, 1, 1);
 
     QFrame *wdg_proc_fr = new QFrame(proc_split);
     wdg_proc_fr->setFrameShape(QFrame::StyledPanel);
@@ -1098,7 +1131,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
 
     lab = new QLabel(_("Procedure language:"),wdg_proc_fr);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred));
-    glay->addWidget(lab,1,0);
+    glay->addWidget(lab, 1, 0);
     proc_lang = new QComboBox(wdg_proc_fr);
     proc_lang->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred));
     proc_lang->setObjectName("/proc/calc/progLng");
@@ -1107,11 +1140,11 @@ VisItProp::VisItProp( VisDevelop *parent ) :
 
     lab = new QLabel(_("Period of the calculating, milliseconds:"),wdg_proc_fr);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
-    glay->addWidget(lab,1,2);
-    proc_per = new LineEdit(wdg_proc_fr);
+    glay->addWidget(lab, 1, 2);
+    proc_per = new LineEdit(wdg_proc_fr, LineEdit::Text, false, false);
     proc_per->setObjectName("/proc/calc/per");
     connect(proc_per, SIGNAL(apply()), this, SLOT(isModify()));
-    glay->addWidget(proc_per,1,3);
+    glay->addWidget(proc_per, 1, 3);
 
     lab = new QLabel(_("Translate:"),wdg_proc_fr);
     lab->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Preferred));
@@ -1124,6 +1157,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     proc_text = new TextEdit(wdg_proc_fr);
     proc_text->setObjectName("/proc/calc/prog");
     connect(proc_text, SIGNAL(apply()), this, SLOT(isModify()));
+    connect(proc_text, SIGNAL(textChanged(const QString&)), this, SLOT(progChanged()));
     glay->addWidget(proc_text, 2, 0, 1, 6);
 
     //Add tab 'Links'
@@ -1198,6 +1232,12 @@ void VisItProp::showDlg( const string &iit, bool reload )
     if(gnd) {
 	wdg_tabs->setTabText(0,gnd->attr("dscr").c_str());
 
+	// Status
+	gnd = TCntrNode::ctrId(root, obj_st->objectName().toStdString(), true);
+	if(gnd) {
+	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_st->objectName().toStdString(),TSYS::PathEl));
+	    if(!owner()->cntrIfCmd(req)) obj_st->setText(req.text().c_str());
+	}else obj_st->setText("");
 	// Enable stat
 	gnd = TCntrNode::ctrId(root,obj_enable->objectName().toStdString(),true);
 	obj_enable->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
@@ -1210,7 +1250,7 @@ void VisItProp::showDlg( const string &iit, bool reload )
 	obj_parent->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
 	if(gnd) selectParent();
 	// Used
-	gnd = TCntrNode::ctrId(root, obj_used->objectName().toStdString(), true);
+	/*gnd = TCntrNode::ctrId(root, obj_used->objectName().toStdString(), true);
 	if(gnd) {
 	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_used->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_used->setText(req.text().c_str());
@@ -1220,7 +1260,7 @@ void VisItProp::showDlg( const string &iit, bool reload )
 	if(gnd) {
 	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_tmstmp->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_tmstmp->setText(atm2s(s2i(req.text())).c_str());
-	}else obj_tmstmp->setText("");
+	}else obj_tmstmp->setText("");*/
 
 	// Icon
 	gnd = TCntrNode::ctrId(root,obj_ico->objectName().toStdString(),true);
@@ -1262,8 +1302,12 @@ void VisItProp::showDlg( const string &iit, bool reload )
 	gnd = TCntrNode::ctrId(root,obj_descr->objectName().toStdString(),true);
 	obj_descr->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
 	if(gnd) {
-	    req.clear();
-	    req.setAttr("path",ed_it+"/"+TSYS::strEncode(obj_descr->objectName().toStdString(),TSYS::PathEl));
+	    if(s2i(gnd->attr("cols"))) {
+		obj_descr->workWdg()->setWordWrapMode(QTextOption::WordWrap);
+		obj_descr->workWdg()->setLineWrapMode(QTextEdit::FixedColumnWidth);
+		obj_descr->workWdg()->setLineWrapColumnOrWidth(s2i(gnd->attr("cols")));
+	    }
+	    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(obj_descr->objectName().toStdString(),TSYS::PathEl));
 	    if(!owner()->cntrIfCmd(req)) obj_descr->setText(req.text().c_str());
 	}
 
@@ -1289,17 +1333,21 @@ void VisItProp::showDlg( const string &iit, bool reload )
 
     //Attributes dialog's page
     gnd = TCntrNode::ctrId(root,"/attr",true);
-    wdg_tabs->setTabEnabled(1,gnd);
-    if(gnd) wdg_tabs->setTabText(1,gnd->attr("dscr").c_str());
+    wdg_tabs->setTabEnabled(1, gnd);
+    if(gnd) wdg_tabs->setTabText(1, gnd->attr("dscr").c_str());
 
     gnd = TCntrNode::ctrId(root,"/links",true);
-    wdg_tabs->setTabEnabled(3,gnd);
-    if(gnd) wdg_tabs->setTabText(3,gnd->attr("dscr").c_str());
+    wdg_tabs->setTabEnabled(3, gnd);
+    if(gnd) wdg_tabs->setTabText(3, gnd->attr("dscr").c_str());
 
     //Process dialog's page
-    gnd=TCntrNode::ctrId(root,"/proc",true);
-    wdg_tabs->setTabEnabled(2,gnd);
-    if(gnd) wdg_tabs->setTabText(2,gnd->attr("dscr").c_str());
+    gnd = TCntrNode::ctrId(root,"/proc",true);
+    wdg_tabs->setTabEnabled(2, gnd);
+    if(gnd) wdg_tabs->setTabText(2, gnd->attr("dscr").c_str());
+
+    proc_per->setValue("");
+    proc_text->setText("");
+
     is_modif = false;
 
     //Show dialog
@@ -1408,16 +1456,19 @@ void VisItProp::tabChanged( int itb )
 	    if(obj_attr_cfg->topLevelItemCount()) obj_attr_cfg->topLevelItem(0)->setData(0,Qt::UserRole+1,atypes);
 
 	    //  Calculate period
-	    gnd = TCntrNode::ctrId(root,proc_per->objectName().toStdString(),true);
-	    proc_per->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
-	    if(gnd) {
-		req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(proc_per->objectName().toStdString(),TSYS::PathEl));
-		if(!owner()->cntrIfCmd(req)) proc_per->setValue(req.text().c_str());
+	    if(!proc_per->isEdited()) {
+		gnd = TCntrNode::ctrId(root,proc_per->objectName().toStdString(),true);
+		proc_per->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
+		if(gnd) {
+		    req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(proc_per->objectName().toStdString(),TSYS::PathEl));
+		    if(!owner()->cntrIfCmd(req)) proc_per->setValue(req.text().c_str());
+		}
 	    }
 	    //  Calc language
-	    gnd = TCntrNode::ctrId(root,proc_lang->objectName().toStdString(),true);
+	    gnd = TCntrNode::ctrId(root,proc_lang->objectName().toStdString(), true);
 	    proc_lang->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
 	    if(gnd) {
+		proc_lang->setToolTip(gnd->attr("help").c_str());
 		req.clear()->setAttr("path",ed_it+"/"+TSYS::strEncode(proc_lang->objectName().toStdString(),TSYS::PathEl));
 		if(!owner()->cntrIfCmd(req)) sval = req.text().c_str();
 		//   Get combo list
@@ -1438,15 +1489,22 @@ void VisItProp::tabChanged( int itb )
 		if(!owner()->cntrIfCmd(req)) proc_text_tr->setChecked(s2i(req.text()));
 	    }
 	    //  Calc procedure
-	    gnd = TCntrNode::ctrId(root,proc_text->objectName().toStdString(),true);
-	    proc_text->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
-	    if(gnd) {
-		req.clear()->setName("CntrReqs")->setAttr("path",ed_it);
-		req.childAdd("get")->setAttr("path",TSYS::strEncode(proc_text->objectName().toStdString(),TSYS::PathEl));
-		req.childAdd("SnthHgl")->setAttr("path",TSYS::strEncode(proc_text->objectName().toStdString(),TSYS::PathEl));
-		if(!owner()->cntrIfCmd(req)) {
-		    proc_text->setText(req.childGet(0)->text().c_str());
-		    proc_text->setSnthHgl(*req.childGet(1));
+	    if(!proc_text->isEdited()) {
+		gnd = TCntrNode::ctrId(root,proc_text->objectName().toStdString(),true);
+		proc_text->setEnabled(gnd && s2i(gnd->attr("acs"))&SEC_WR);
+		if(gnd) {
+		    req.clear()->setName("CntrReqs")->setAttr("path",ed_it);
+		    req.childAdd("get")->setAttr("path",TSYS::strEncode(proc_text->objectName().toStdString(),TSYS::PathEl));
+		    req.childAdd("SnthHgl")->setAttr("path",TSYS::strEncode(proc_text->objectName().toStdString(),TSYS::PathEl));
+		    if(!owner()->cntrIfCmd(req)) {
+			proc_text->blockSignals(true);
+			proc_text->setSnthHgl(*req.childGet(1));
+			proc_text->setText(req.childGet(0)->text().c_str());
+			proc_text->setProperty("inherited", (bool)s2i(req.childGet(0)->attr("inherited")));
+			proc_text->setProperty("redefined", (bool)s2i(req.childGet(0)->attr("redefined")));
+			proc_text->setProperty("redefAccept", false);
+			proc_text->blockSignals(false);
+		    }
 		}
 	    }
 
@@ -1457,6 +1515,17 @@ void VisItProp::tabChanged( int itb )
     }
 }
 
+void VisItProp::progChanged( )
+{
+    TextEdit *et = (TextEdit*)sender();
+    if(et->property("inherited").toBool() && !et->property("redefined").toBool() && !et->property("redefAccept").toBool()) {
+	InputDlg dlg(this, windowIcon(), _("Are you sure of editing the inherited procedure, since that can cause for unexpectedly loss of the access to the original one?!"),
+	    _("Editing an inherited procedure"), false, false);
+	if(dlg.exec() == QDialog::Accepted) et->setProperty("redefAccept", true);
+	else et->cancelSlot();
+    }
+}
+
 void VisItProp::selectParent( )
 {
     XMLNode req("get");
@@ -1556,14 +1625,14 @@ void VisItProp::closeEvent( QCloseEvent *ce )
     QList<LineEdit*> lnEdWs = findChildren<LineEdit*>();
     QList<TextEdit*> txtEdWs = findChildren<TextEdit*>();
     bool notApplyPresent = false;
-    for(int i_it = 0; !notApplyPresent && i_it < lnEdWs.size(); ++i_it) notApplyPresent = lnEdWs[i_it]->isEdited();
-    for(int i_it = 0; !notApplyPresent && i_it < txtEdWs.size(); ++i_it) notApplyPresent = txtEdWs[i_it]->isEdited();
+    for(int iIt = 0; !notApplyPresent && iIt < lnEdWs.size(); ++iIt)  notApplyPresent = lnEdWs[iIt]->isEdited();
+    for(int iIt = 0; !notApplyPresent && iIt < txtEdWs.size(); ++iIt) notApplyPresent = txtEdWs[iIt]->isEdited();
 
     if(notApplyPresent && QMessageBox::information(this,_("Saving the changes"),_("Some changes were made!\nSave the changes to the DB before the closing?"),
 	    QMessageBox::Apply|QMessageBox::Cancel,QMessageBox::Apply) == QMessageBox::Apply)
     {
-	for(int i_it = 0; i_it < lnEdWs.size(); ++i_it) if(lnEdWs[i_it]->isEdited()) isModify(lnEdWs[i_it]);
-	for(int i_it = 0; i_it < txtEdWs.size(); ++i_it) if(txtEdWs[i_it]->isEdited()) isModify(txtEdWs[i_it]);
+	for(int iIt = 0; iIt < lnEdWs.size(); ++iIt) if(lnEdWs[iIt]->isEdited()) isModify(lnEdWs[iIt]);
+	for(int iIt = 0; iIt < txtEdWs.size(); ++iIt) if(txtEdWs[iIt]->isEdited()) isModify(txtEdWs[iIt]);
     }
 
     if(is_modif) emit apply(ed_it);
@@ -1574,6 +1643,12 @@ void VisItProp::closeEvent( QCloseEvent *ce )
     ce->accept();
 }
 
+void VisItProp::keyPressEvent( QKeyEvent *ce )
+{
+    if(ce->key() == Qt::Key_Escape) ce->ignore();
+    else QDialog::keyPressEvent(ce);
+}
+
 void VisItProp::showEvent( QShowEvent * event )
 {
     QSize src = size();
