@@ -1,7 +1,7 @@
 
 //OpenSCADA file: tprmtmpl.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2018 by Roman Savochenko, <rom_as@oscada.org>      *
+ *   Copyright (C) 2003-2020 by Roman Savochenko, <roman@oscada.org>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -57,7 +57,7 @@ void TPrmTempl::postEnable( int flag )
 {
     //Create default IOs
     if(flag&TCntrNode::NodeConnect) {
-	ioIns(new IO("f_frq",_("Frequency of calculation of the function (Hz)"),IO::Real,TPrmTempl::LockAttr,"1000",false), 0);
+	ioIns(new IO("f_frq",_("Frequency of calculation of the function, Hz"),IO::Real,TPrmTempl::LockAttr,"1000",false), 0);
 	ioIns(new IO("f_start",_("Function start flag"),IO::Boolean,TPrmTempl::LockAttr,"0",false), 1);
 	ioIns(new IO("f_stop",_("Function stop flag"),IO::Boolean,TPrmTempl::LockAttr,"0",false), 2);
 	ioIns(new IO("f_err",_("Function error"),IO::String,TPrmTempl::LockAttr,"0",false), 3);
@@ -130,7 +130,7 @@ void TPrmTempl::setStart( bool vl )
 	    if(ioIds.find(io(id)->id()) != ioIds.end()) { ioDel(id); modif(); }
 	    else { ioIds[io(id)->id()] = true; id++; }
 
-	//Compile new function
+	//Compile the new function
 	if(prog().size())
 	    work_prog = SYS->daq().at().at(TSYS::strSepParse(progLang(),0,'.')).at().
 					compileFunc(TSYS::strSepParse(progLang(),1,'.'),*this,prog(),"",maxCalcTm());
@@ -207,15 +207,15 @@ void TPrmTempl::save_( )
     //Save IO
     TConfig cfg(&owner().owner().elTmplIO());
     cfg.cfg("TMPL_ID").setS(id(),true);
-    for(int i_io = 0; i_io < ioSize(); i_io++) {
-	if(io(i_io)->flg()&TPrmTempl::LockAttr) continue;
-	cfg.cfg("ID").setS(io(i_io)->id());
-	cfg.cfg("NAME").setS(io(i_io)->name());
-	cfg.cfg("TYPE").setI(io(i_io)->type());
-	cfg.cfg("FLAGS").setI(io(i_io)->flg());
-	cfg.cfg("VALUE").setNoTransl(!(io(i_io)->type()==IO::String || io(i_io)->flg()&TPrmTempl::CfgLink));
-	cfg.cfg("VALUE").setS(io(i_io)->def());
-	cfg.cfg("POS").setI(i_io);
+    for(int iIO = 0; iIO < ioSize(); iIO++) {
+	if(io(iIO)->flg()&TPrmTempl::LockAttr) continue;
+	cfg.cfg("ID").setS(io(iIO)->id());
+	cfg.cfg("NAME").setS(io(iIO)->name());
+	cfg.cfg("TYPE").setI(io(iIO)->type());
+	cfg.cfg("FLAGS").setI(io(iIO)->flg());
+	cfg.cfg("VALUE").setNoTransl(!(io(iIO)->type()==IO::String || io(iIO)->flg()&(TPrmTempl::CfgLink|IO::Selectable)));
+	cfg.cfg("VALUE").setS(io(iIO)->def());
+	cfg.cfg("POS").setI(iIO);
 	SYS->db().at().dataSet(w_db+"_io",w_cfgpath+"_io",cfg);
     }
     //Clear IO
@@ -233,7 +233,7 @@ void TPrmTempl::save_( )
 TVariant TPrmTempl::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
 {
     //Configuration functions call
-    TVariant cfRez = objFunc(iid, prms, user);
+    TVariant cfRez = objFunc(iid, prms, user, RWRWR_, "root:" SDAQ_ID);
     if(!cfRez.isNull()) return cfRez;
 
     return TCntrNode::objFuncCall(iid, prms, user);
@@ -244,7 +244,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
     //Get page info
     if(opt->name() == "info") {
 	TCntrNode::cntrCmdProc(opt);
-	ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter template: ")+name(),RWRWR_,"root",SDAQ_ID,1,"doc","Documents/User_API|Documents/User_API");
+	ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter template: ")+name(),RWRWR_,"root",SDAQ_ID,1,"doc","User_API|Documents/User_API");
 	if(ctrMkNode("area",opt,-1,"/tmpl",_("Template"))) {
 	    if(ctrMkNode("area",opt,-1,"/tmpl/st",_("State"))) {
 		ctrMkNode("fld",opt,-1,"/tmpl/st/st",_("Accessing"),RWRWR_,"root",SDAQ_ID,1,"tp","bool");
@@ -263,8 +263,10 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/io/io/0",_("Identifier"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("list",opt,-1,"/io/io/1",_("Name"),RWRWR_,"root",SDAQ_ID,1,"tp","str");
 		ctrMkNode("list",opt,-1,"/io/io/2",_("Type"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
-		    "sel_id",TSYS::strMess("%d;%d;%d;%d;%d;%d",IO::Real,IO::Integer,IO::Boolean,IO::String,IO::String|(IO::FullText<<8),IO::Object).c_str(),
-		    "sel_list",_("Real;Integer;Boolean;String;Text;Object"));
+		    "sel_id",TSYS::strMess("%d;%d;%d;%d;%d;%d;%d;%d;%d",
+			IO::Real,IO::Integer,IO::Boolean,IO::String,IO::String|(IO::FullText<<8),IO::Object,
+			IO::Integer|(IO::Selectable<<8),IO::Real|(IO::Selectable<<8),IO::String|(IO::Selectable<<8)).c_str(),
+		    "sel_list",_("Real;Integer;Boolean;String;Text;Object;Integer numbers selection;Real numbers selection;Strings selection"));
 		ctrMkNode("list",opt,-1,"/io/io/3",_("Mode"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
 		    "sel_id",TSYS::strMess("%d;%d",IO::Default,IO::Output).c_str(),"sel_list",_("Input;Output"));
 		ctrMkNode("list",opt,-1,"/io/io/4",_("Attribute"),RWRWR_,"root",SDAQ_ID,5,"tp","dec","idm","1","dest","select",
@@ -317,7 +319,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	    for(int id = 0; id < ioSize(); id++) {
 		if(n_id)	n_id->childAdd("el")->setText(io(id)->id());
 		if(n_nm)	n_nm->childAdd("el")->setText(io(id)->name());
-		if(n_type)	n_type->childAdd("el")->setText(i2s(io(id)->type()|((io(id)->flg()&IO::FullText)<<8)));
+		if(n_type)	n_type->childAdd("el")->setText(i2s(io(id)->type()|((io(id)->flg()&(IO::FullText|IO::Selectable))<<8)));
 		if(n_mode)	n_mode->childAdd("el")->setText(i2s(io(id)->flg()&(IO::Output|IO::Return)));
 		if(n_attr)	n_attr->childAdd("el")->setText(i2s(io(id)->flg()&(TPrmTempl::AttrRead|TPrmTempl::AttrFull)));
 		if(n_accs)	n_accs->childAdd("el")->setText(i2s(io(id)->flg()&(TPrmTempl::CfgConst|TPrmTempl::CfgLink)));
@@ -352,11 +354,14 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	    if((col == 0 || col == 1) && !opt->text().size())	throw err_sys(_("Empty value is not allowed."));
 	    modif();
 	    switch(col) {
-		case 0:	io(row)->setId(opt->text());	break;
-		case 1:	io(row)->setName(opt->text());	break;
+		case 0:
+		    opt->setText(TSYS::strEncode(sTrm(opt->text()),TSYS::Limit,i2s(owner().owner().elTmplIO().fldAt(owner().owner().elTmplIO().fldId("ID")).len())));
+		    io(row)->setId(opt->text());
+		    break;
+		case 1:	io(row)->setName(sTrm(opt->text()));	break;
 		case 2:
 		    io(row)->setType((IO::Type)(s2i(opt->text())&0xFF));
-		    io(row)->setFlg(io(row)->flg()^((io(row)->flg()^(s2i(opt->text())>>8))&IO::FullText));
+		    io(row)->setFlg(io(row)->flg()^((io(row)->flg()^(s2i(opt->text())>>8))&(IO::FullText|IO::Selectable)));
 		    break;
 		case 3:	io(row)->setFlg(io(row)->flg()^((io(row)->flg()^s2i(opt->text()))&(IO::Output|IO::Return)));		break;
 		case 4:	io(row)->setFlg(io(row)->flg()^((io(row)->flg()^s2i(opt->text()))&(TPrmTempl::AttrRead|TPrmTempl::AttrFull)));		break;
@@ -385,145 +390,261 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
     else TCntrNode::cntrCmdProc(opt);
 }
 
-TPrmTempl::Impl::Impl( TCntrNode *iobj, const string &iname ) : TValFunc(iname.c_str(),NULL,false), obj(iobj)
+TPrmTempl::Impl::Impl( TCntrNode *iobj, const string &iname, bool blked ) : TValFunc(iname.c_str(),NULL,blked), obj(iobj)
 {
 
 }
 
+int  TPrmTempl::Impl::lnkId( const string &nm )
+{
+    MtxAlloc res(lnkRes, true);
+    for(int iIO = 0; iIO < func()->ioSize(); iIO++)
+	if(func()->io(iIO)->id() == nm)
+	    return iIO;
+    return -1;
+}
+
 bool TPrmTempl::Impl::lnkPresent( int num )
 {
-    MtxAlloc res(obj->dataRes(), true);
+    MtxAlloc res(lnkRes, true);
     map<int,SLnk>::iterator it = lnks.find(num);
     return (it != lnks.end());
 }
 
+void TPrmTempl::Impl::lnkList( vector<int> &ls ) const
+{
+    MtxAlloc res(const_cast<ResMtx&>(lnkRes), true);
+    for(map<int,SLnk>::const_iterator iL = lnks.begin(); iL != lnks.end(); ++iL)
+	ls.push_back(iL->first);
+}
+
 void TPrmTempl::Impl::lnkAdd( int num, const SLnk &l )
 {
-    obj->dataRes().lock();
+    lnkRes.lock();
     lnks[num] = l;
-    obj->dataRes().unlock();
+    lnkRes.unlock();
 }
 
-string TPrmTempl::Impl::lnkAttr( int num )
+string TPrmTempl::Impl::lnkAddr( int num, bool spec ) const
 {
-    MtxAlloc res(obj->dataRes(), true);
+    MtxAlloc res(const_cast<ResMtx&>(lnkRes), true);
+    map<int,SLnk>::const_iterator it = lnks.find(num);
+    if(it == lnks.end()) throw TError(obj->nodePath().c_str(), _("Error of parameter ID."));
+    return spec ? it->second.addrSpec : it->second.addr;
+}
+
+void TPrmTempl::Impl::lnkAddrSet( int num, const string &vl, bool spec )
+{
+    MtxAlloc res(lnkRes, true);
     map<int,SLnk>::iterator it = lnks.find(num);
     if(it == lnks.end()) throw TError(obj->nodePath().c_str(), _("Error of parameter ID."));
-    return it->second.prmAttr;
+    if(spec)	it->second.addrSpec = vl;
+    else if(it->second.addr != vl) {
+	it->second.addr = vl;
+	it->second.con.free(); it->second.addrSpec = "";	//for reconnect
+    }
 }
 
-void TPrmTempl::Impl::lnkAttrSet( int num, const string &vl )
+bool TPrmTempl::Impl::lnkInit( int num, bool toRecnt )
 {
-    MtxAlloc res(obj->dataRes(), true);
+    MtxAlloc res(lnkRes, true);
     map<int,SLnk>::iterator it = lnks.find(num);
-    if(it == lnks.end()) throw TError(obj->nodePath().c_str(), _("Error of parameter ID."));
-    it->second.prmAttr = vl;
-}
+    if(it == lnks.end()) return false;
+    if(toRecnt) { it->second.con.free(); it->second.addrSpec = ""; }
 
-void TPrmTempl::Impl::lnkClear( bool andFunc )
-{
-    obj->dataRes().lock();
-    lnks.clear();
-    if(andFunc) setFunc(NULL);
-    obj->dataRes().unlock();
-}
+    bool toCheckRes = true;
 
-bool TPrmTempl::Impl::initTmplLnks( bool checkNoLink )
-{
-    //Init links
-    string nmod, ncntr, nprm, nattr;
+retry:
+    if(it->second.addr.empty() || it->second.addr.compare(0,4,"val:") == 0)	toCheckRes = false;
+    // Try to relink
+    else if(it->second.con.freeStat()) {
+	string addr = it->second.addr, aAddr;
+	res.unlock();
+
+	AutoHD<TVal> con;
+	int objOff = 0;
 
-    MtxAlloc res(obj->dataRes(), true);
-    for(map<int,SLnk>::iterator iL = lnks.begin(); iL != lnks.end(); ++iL) {
-	if(checkNoLink && !iL->second.aprm.freeStat()) continue;
 	try {
-	    iL->second.aprm.free();
-	    iL->second.detOff = 0;
-	    iL->second.aprm = SYS->daq().at().attrAt(TSYS::strParse(iL->second.prmAttr,0,"#",&iL->second.detOff), '.', true);
-	    if(!iL->second.aprm.freeStat()) {
-		if(iL->second.aprm.at().fld().type() == TFld::Object && iL->second.detOff < (int)iL->second.prmAttr.size())
-		    setS(iL->first, iL->second.aprm.at().getO().at().propGet(iL->second.prmAttr.substr(iL->second.detOff),'.'));
-		else setS(iL->first, iL->second.aprm.at().getS());
+	    con = SYS->daq().at().attrAt((aAddr=TSYS::strParse(addr,0,"#",&objOff)), '.', true);
+	    if(!con.freeStat()) {
+		if(con.at().fld().type() == TFld::Object && objOff < (int)addr.size())
+		    setS(num, con.at().getO().at().propGet(addr.substr(objOff),'.'));
+		else setS(num, con.at().getS());
+		toCheckRes = false;
 	    }
-	    else return true;
-	} catch(TError &err) { return true; }
+	    else if(!SYS->daq().at().prmAt(aAddr,'.',true).freeStat()) toCheckRes = false;	//just a parameter
+	} catch(TError &err) { }
+
+	//Set the connection result
+	res.lock();
+	// Relink after going behind the lock
+	if((it=lnks.find(num)) == lnks.end())	return false;
+	if(it->second.addr != addr) goto retry;			//the item address changed
+
+	it->second.con = con;
+	it->second.objOff = objOff;
     }
 
-    return false;
+    return toCheckRes;
 }
 
-void TPrmTempl::Impl::inputLinks( )
+bool TPrmTempl::Impl::lnkActive( int num )
 {
-    MtxAlloc res(obj->dataRes(), true);
-    for(map<int,SLnk>::iterator iL = lnks.begin(); iL != lnks.end(); ++iL)
-	if(iL->second.aprm.freeStat()) setS(iL->first, EVAL_STR);
-	else if(iL->second.aprm.at().fld().type() == TFld::Object && iL->second.detOff < (int)iL->second.prmAttr.size())
-	    set(iL->first, iL->second.aprm.at().getO().at().propGet(iL->second.prmAttr.substr(iL->second.detOff),'.'));
-	else set(iL->first, iL->second.aprm.at().get());
+    MtxAlloc res(lnkRes, true);
+    map<int,SLnk>::iterator it = lnks.find(num);
+    return (it != lnks.end() && !it->second.con.freeStat());
 }
 
-void TPrmTempl::Impl::outputLinks( )
+TVariant TPrmTempl::Impl::lnkInput( int num )
 {
-    MtxAlloc res(obj->dataRes(), true);
-    for(map<int,SLnk>::iterator iL = lnks.begin(); iL != lnks.end(); ++iL)
-	if(!iL->second.aprm.freeStat() && ioMdf(iL->first))
-	    outputLink(iL->first, get(iL->first));
+    MtxAlloc res(lnkRes, true);
+    map<int,SLnk>::iterator it = lnks.find(num);
 
-	/*if(!iL->second.aprm.freeStat() && ioMdf(iL->first) &&
-		ioFlg(iL->first)&(IO::Output|IO::Return) && !(iL->second.aprm.at().fld().flg()&TFld::NoWrite))
-	{
-	    TVariant vl = get(iL->first);
-	    if(!vl.isEVal()) {
-		if(iL->second.aprm.at().fld().type() == TFld::Object && iL->second.detOff < (int)iL->second.prmAttr.size()) {
-		    iL->second.aprm.at().getO().at().propSet(iL->second.prmAttr.substr(iL->second.detOff), '.', vl);
-		    iL->second.aprm.at().setO(iL->second.aprm.at().getO());	//For modify object sign
-		}
-		else iL->second.aprm.at().set(vl);
-	    }
-	}*/
+    if(it == lnks.end())	return EVAL_REAL;
+    if(it->second.addr.compare(0,4,"val:") == 0)return it->second.addr.substr(4);
+    if(it->second.con.freeStat())		return EVAL_REAL;
+    AutoHD<TVal> con = it->second.con;
+    int objOff = it->second.objOff;
+    string addr = it->second.addr;
+    res.unlock();
+
+    if(con.at().fld().type() == TFld::Object && objOff < (int)addr.size())
+	return con.at().getO().at().propGet(addr.substr(objOff),'.');
+
+    return con.at().get();
 }
 
-bool TPrmTempl::Impl::outputLink( int num, const TVariant &vl )
+bool TPrmTempl::Impl::lnkOutput( int num, const TVariant &vl )
 {
     if(vl.isEVal()) return false;
-    MtxAlloc res(obj->dataRes(), true);
+    MtxAlloc res(lnkRes, true);
     map<int,SLnk>::iterator it = lnks.find(num);
-    if(it == lnks.end() || it->second.aprm.freeStat() || (it->second.aprm.at().fld().flg()&TFld::NoWrite) ||
+    if(it == lnks.end() || it->second.con.freeStat() || (it->second.con.at().fld().flg()&TFld::NoWrite) ||
 	    !(ioFlg(num)&(IO::Output|IO::Return)))
 	return false;
-    if(it->second.aprm.at().fld().type() == TFld::Object && it->second.detOff < (int)it->second.prmAttr.size()) {
-	it->second.aprm.at().getO().at().propSet(it->second.prmAttr.substr(it->second.detOff), '.', vl);
-	it->second.aprm.at().setO(it->second.aprm.at().getO());	//For modify object sign
-    }
-    else it->second.aprm.at().set(vl);
+    AutoHD<TVal> con = it->second.con;
+    int objOff = it->second.objOff;
+    string addr = it->second.addr;
+    res.unlock();
+
+    if(con.at().fld().type() == TFld::Object && objOff < (int)addr.size()) {
+	con.at().getO().at().propSet(addr.substr(objOff), '.', vl);
+	con.at().setO(con.at().getO());	//For the modifying object sign
+    } else con.at().set(vl);
 
     return true;
 }
 
-bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt )
+void TPrmTempl::Impl::addLinksAttrs( TElem *attrsCntr )
 {
-    MtxAlloc res(obj->dataRes(), true);
+    map<string, bool> als;
+
+    //MtxAlloc res(lnkRes, true);	//!!!! There is not a direct access to links
+    for(int iIO = 0; iIO < func()->ioSize(); iIO++) {
+	if((func()->io(iIO)->flg()&TPrmTempl::CfgLink) && !lnkPresent(iIO)) {
+	    lnkAdd(iIO, TPrmTempl::Impl::SLnk());
+	    setS(iIO, EVAL_STR);	//|| setS(iIO,"0"); on DAQ.ModBus
+	}
+	if(attrsCntr && (func()->io(iIO)->flg()&(TPrmTempl::AttrRead|TPrmTempl::AttrFull))) {
+	    unsigned fId = 0;
+	    unsigned flg = TVal::DirWrite|TVal::DirRead;
+	    if(func()->io(iIO)->flg()&IO::FullText)		flg |= TFld::FullText;
+	    string selVals, selNms;
+	    if(func()->io(iIO)->flg()&IO::Selectable) {
+		flg |= TFld::Selectable;
+		selVals = TSYS::strLine(func()->io(iIO)->def(), 1);
+		selNms = TSYS::strLine(func()->io(iIO)->def(), 2);
+	    }
+	    if(func()->io(iIO)->flg()&TPrmTempl::AttrRead)	flg |= TFld::NoWrite;
+	    TFld::Type tp = TFld::type(ioType(iIO));
+	    if((fId=attrsCntr->fldId(func()->io(iIO)->id(),true)) < attrsCntr->fldSize()) {
+		if(attrsCntr->fldAt(fId).type() != tp)
+		    try{ attrsCntr->fldDel(fId); }
+		    catch(TError &err){ mess_warning(err.cat.c_str(),err.mess.c_str()); }
+		else {
+		    attrsCntr->fldAt(fId).setFlg(flg);
+		    attrsCntr->fldAt(fId).setDescr(func()->io(iIO)->name().c_str());
+		    if((flg&TFld::Selectable)) {
+			attrsCntr->fldAt(fId).setValues(selVals);
+			attrsCntr->fldAt(fId).setSelNames(selNms);
+		    }
+		}
+	    }
+
+	    if(!attrsCntr->fldPresent(func()->io(iIO)->id()))
+		attrsCntr->fldAdd(new TFld(func()->io(iIO)->id().c_str(),func()->io(iIO)->name().c_str(),tp,flg,"","",selVals.c_str(),selNms.c_str()));
+
+	    als[func()->io(iIO)->id()] = true;
+	}
+    }
+
+    //Check for removing
+    for(int iP = 0; attrsCntr && iP < (int)attrsCntr->fldSize(); iP++)
+	if(als.find(attrsCntr->fldAt(iP).name()) == als.end())
+	    try{ attrsCntr->fldDel(iP); iP--; }
+	    catch(TError &err) { mess_warning(err.cat.c_str(),err.mess.c_str()); }
+}
+
+bool TPrmTempl::Impl::initLnks( bool toRecnt )
+{
+    vector<int> ls;
+    bool chkLnkNeed = false;
+
+    lnkList(ls);
+    for(int iL = 0; iL < (int)ls.size(); ++iL)
+	if(lnkInit(ls[iL],toRecnt)) chkLnkNeed = true;
+
+    return chkLnkNeed;
+}
+
+void TPrmTempl::Impl::cleanLnks( bool andFunc )
+{
+    lnkRes.lock();
+    lnks.clear();
+    if(andFunc) setFunc(NULL);
+    lnkRes.unlock();
+}
+
+void TPrmTempl::Impl::inputLinks( )
+{
+    vector<int> ls;
+    lnkList(ls);
+    for(int iL = 0; iL < (int)ls.size(); ++iL)
+	set(ls[iL], lnkInput(ls[iL]));
+}
+
+void TPrmTempl::Impl::outputLinks( )
+{
+    vector<int> ls;
+    lnkList(ls);
+    for(int iL = 0; iL < (int)ls.size(); ++iL)
+	if(ioMdf(ls[iL])) lnkOutput(ls[iL], get(ls[iL]));
+}
+
+bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt, const string &pref )
+{
+    MtxAlloc res(lnkRes, true);
     //Get page info
-    if(opt->name() == "info" && ctrMkNode("area",opt,-1,"/cfg",_("Template configuration"))) {
+    if(opt->name() == "info" && ctrMkNode("area",opt,-1,pref.c_str(),_("Template configuration"))) {
 	vector<string> list;
-	ctrMkNode("fld",opt,-1,"/cfg/attr_only",_("Only attributes are to be shown"),RWRWR_,"root",SDAQ_ID,1,"tp","bool");
-	if(ctrMkNode("area",opt,-1,"/cfg/prm",_("Parameters")))
+	ctrMkNode("fld",opt,-1,(pref+"/attr_only").c_str(),_("Show attributes"),RWRWR_,"root",SDAQ_ID,1,"tp","bool");
+	if(ctrMkNode("area",opt,-1,(pref+"/prm").c_str(),_("Parameters")))
 	    for(int iIO = 0; iIO < ioSize(); iIO++) {
 		if(!(func()->io(iIO)->flg()&(TPrmTempl::CfgLink|TPrmTempl::CfgConst)))
 		    continue;
-		// Check select param
+		// Check the selected param
 		bool is_lnk = func()->io(iIO)->flg()&TPrmTempl::CfgLink;
-		if(is_lnk && func()->io(iIO)->def().size() &&
+		if(is_lnk && TSYS::strLine(func()->io(iIO)->def(),0).size() &&
 		    !s2i(TBDS::genDBGet(obj->nodePath()+"onlAttr","0",opt->attr("user"))))
 		{
-		    string nprm = TSYS::strSepParse(func()->io(iIO)->def(),0,'|');
+		    string nprm = TSYS::strSepParse(TSYS::strLine(func()->io(iIO)->def(),0),0,'|');
 		    // Check already to present parameters
 		    bool f_ok = false;
 		    for(unsigned iL = 0; iL < list.size() && !f_ok; iL++)
 			if(list[iL] == nprm) f_ok = true;
 		    if(!f_ok) {
-			ctrMkNode("fld",opt,-1,(string("/cfg/prm/pr_")+i2s(iIO)).c_str(),nprm,RWRWR_,"root",SDAQ_ID,
-			    3,"tp","str","dest","sel_ed","select",(string("/cfg/prm/pl_")+i2s(iIO)).c_str());
+			ctrMkNode("fld",opt,-1,(pref+"/prm/pr_"+i2s(iIO)).c_str(),nprm,RWRWR_,"root",SDAQ_ID,
+			    4, "tp","str", "dest","sel_ed", "select",(pref+"/prm/pl_"+i2s(iIO)).c_str(), "help",lnkHelp().c_str());
 			list.push_back(nprm);
 		    }
 		}
@@ -533,16 +654,16 @@ bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt )
 		    if(!is_lnk)
 			switch(ioType(iIO)) {
 			    case IO::Integer:	tip = "dec";	break;
-			    case IO::Real:		tip = "real";	break;
+			    case IO::Real:	tip = "real";	break;
 			    case IO::Boolean:	tip = "bool";	break;
 			    case IO::String:
 				if(func()->io(iIO)->flg()&IO::FullText) fullTxt = true;
 				break;
 			    case IO::Object:	fullTxt = true;	break;
 			}
-		    XMLNode *wn = ctrMkNode("fld",opt,-1,(string("/cfg/prm/el_")+i2s(iIO)).c_str(),
+		    XMLNode *wn = ctrMkNode("fld",opt,-1,(pref+"/prm/el_"+i2s(iIO)).c_str(),
 			    func()->io(iIO)->name(),RWRWR_,"root",SDAQ_ID,1,"tp",tip);
-		    if(wn && is_lnk) wn->setAttr("dest","sel_ed")->setAttr("select","/cfg/prm/ls_"+i2s(iIO));
+		    if(wn && is_lnk) wn->setAttr("dest","sel_ed")->setAttr("select",pref+"/prm/ls_"+i2s(iIO))->setAttr("help",lnkHelp());
 		    if(wn && fullTxt)wn->setAttr("cols","100")->setAttr("rows","4");
 		}
 	    }
@@ -551,13 +672,15 @@ bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt )
 
     //Process command to page
     string a_path = opt->attr("path");
-    if(a_path == "/cfg/attr_only") {
+    if(a_path.find(pref) != 0)	return false;
+    a_path = a_path.substr(pref.size());
+    if(a_path == "/attr_only") {
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD))	opt->setText(TBDS::genDBGet(obj->nodePath()+"onlAttr","0",opt->attr("user")));
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR))	TBDS::genDBSet(obj->nodePath()+"onlAttr",opt->text(),opt->attr("user"));
     }
-    else if(a_path.substr(0,12) == "/cfg/prm/pr_") {
+    else if(a_path.compare(0,8,"/prm/pr_") == 0) {
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD)) {
-	    string lnk_val = lnks[s2i(a_path.substr(12))].prmAttr;
+	    string lnk_val = lnks[s2i(a_path.substr(8))].addr;
 	    if(!SYS->daq().at().attrAt(TSYS::strParse(lnk_val,0,"#"),'.',true).freeStat()) {
 		opt->setText(lnk_val.substr(0,lnk_val.rfind(".")));
 		opt->setText(opt->text()+" (+)");
@@ -567,53 +690,51 @@ bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt )
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR)) {
 	    TParamContr *pC = dynamic_cast<TParamContr*>(obj);
 	    string no_set;
-	    string p_nm = TSYS::strSepParse(func()->io(s2i(a_path.substr(12)))->def(),0,'|');
+	    string p_nm = TSYS::strSepParse(TSYS::strLine(func()->io(s2i(a_path.substr(8)))->def(),0),0,'|');
 	    string p_vl = TSYS::strParse(opt->text(), 0, " ");
 	    if(pC && p_vl == pC->DAQPath()) throw TError(obj->nodePath().c_str(),_("Error, recursive linking."));
 	    AutoHD<TValue> prm = SYS->daq().at().prmAt(p_vl, '.', true);
 
 	    for(map<int,SLnk>::iterator iL = lnks.begin(); iL != lnks.end(); ++iL)
-		if(p_nm == TSYS::strSepParse(func()->io(iL->first)->def(),0,'|')) {
-		    iL->second.prmAttr = p_vl;
-		    string p_attr = TSYS::strSepParse(func()->io(iL->first)->def(),1,'|');
+		if(p_nm == TSYS::strSepParse(TSYS::strLine(func()->io(iL->first)->def(),0),0,'|')) {
+		    lnkAddrSet(iL->first, p_vl);
+		    string p_attr = TSYS::strSepParse(TSYS::strLine(func()->io(iL->first)->def(),0),1,'|');
 		    if(!prm.freeStat()) {
-			if(prm.at().vlPresent(p_attr)) {
-			    iL->second.prmAttr = p_vl+"."+p_attr;
-			    obj->modif();
-			}
+			if(prm.at().vlPresent(p_attr))	lnkAddrSet(iL->first, p_vl+"."+p_attr);
 			else no_set += p_attr+",";
 		    }
 		}
-	    initTmplLnks();
+	    initLnks();
+	    obj->modif();
 	}
     }
-    else if((a_path.compare(0,12,"/cfg/prm/pl_") == 0 || a_path.compare(0,12,"/cfg/prm/ls_") == 0) && ctrChkNode(opt))
+    else if((a_path.compare(0,8,"/prm/pl_") == 0 || a_path.compare(0,8,"/prm/ls_") == 0) && ctrChkNode(opt))
     {
-	bool is_pl = (a_path.compare(0,12,"/cfg/prm/pl_") == 0);
-	string m_prm = lnks[s2i(a_path.substr(12))].prmAttr;
+	bool is_pl = (a_path.compare(0,8,"/prm/pl_") == 0);
+	string m_prm = lnks[s2i(a_path.substr(8))].addr;
 	if(is_pl && !SYS->daq().at().attrAt(m_prm,'.',true).freeStat()) m_prm = m_prm.substr(0,m_prm.rfind("."));
 	SYS->daq().at().ctrListPrmAttr(opt, m_prm, is_pl, '.');
     }
-    else if(a_path.substr(0,12) == "/cfg/prm/el_") {
+    else if(a_path.substr(0,8) == "/prm/el_") {
 	if(ctrChkNode(opt,"get",RWRWR_,"root",SDAQ_ID,SEC_RD)) {
-	    int iIO = s2i(a_path.substr(12));
+	    int iIO = s2i(a_path.substr(8));
 	    if(func()->io(iIO)->flg()&TPrmTempl::CfgLink) {
-		opt->setText(lnks[iIO].prmAttr);
+		opt->setText(lnks[iIO].addr);
 		if(!SYS->daq().at().attrAt(TSYS::strParse(opt->text(),0,"#"),'.',true).freeStat()) opt->setText(opt->text()+" (+)");
 	    }
 	    else if(func()->io(iIO)->flg()&TPrmTempl::CfgConst)
 		opt->setText(getS(iIO));
 	}
 	if(ctrChkNode(opt,"set",RWRWR_,"root",SDAQ_ID,SEC_WR)) {
-	    int iIO = s2i(a_path.substr(12));
+	    int iIO = s2i(a_path.substr(8));
 	    if(func()->io(iIO)->flg()&TPrmTempl::CfgLink) {
 		string a_vl = TSYS::strParse(opt->text(), 0, " ");
 		//if(TSYS::strSepParse(a_vl,0,'.') == owner().owner().modId() &&
 		//	TSYS::strSepParse(a_vl,1,'.') == owner().id() &&
 		//	TSYS::strSepParse(a_vl,2,'.') == id())
 		//    throw TError(nodePath().c_str(),_("Error, recursive linking."));
-		lnks[iIO].prmAttr = a_vl;
-		initTmplLnks();
+		lnkAddrSet(iIO, a_vl);
+		initLnks();
 	    }
 	    else if(func()->io(iIO)->flg()&TPrmTempl::CfgConst) setS(iIO, opt->text());
 	    obj->modif();
@@ -624,11 +745,22 @@ bool TPrmTempl::Impl::cntrCmdProc( XMLNode *opt )
     return true;
 }
 
+string TPrmTempl::Impl::lnkHelp( )
+{
+    return _("Address to the DAQ attribute writes in the form \"{Type}.{Cntr}.{Prm}[.{PrmN}].{Attr}\", where:\n"
+	"    Type,Cntr,Prm,Attr - DAQ: type, controller object, parameter and attribute.\n"
+	"  Link's template, of the column \"Value\" on the template forming side, writes in the form \"{Parameter}|{attribute}\", where:\n"
+	"    Parameter - specifies the parameter's name as the attribute's container;\n"
+	"    attribute - attributes with the equal value <Parameter> will be grouped and will be appointed only by the indication of the attributes' container,\n"
+	"                and the individual attributes will be associated with the attributes of the container in accordance with this field.\n"
+	"Constant link value writes in the form \"val:{Constant}\".\n");
+}
+
 //*************************************************
 //* TPrmTmplLib                                   *
 //*************************************************
 TPrmTmplLib::TPrmTmplLib( const string &id, const string &name, const string &lib_db ) :
-    TConfig(&SYS->daq().at().elLib()), run_st(false), mId(cfg("ID")), work_lib_db(lib_db)
+    TConfig(&SYS->daq().at().elLib()), runSt(false), mId(cfg("ID")), work_lib_db(lib_db)
 {
     mId = id;
     setName( name );
@@ -720,11 +852,11 @@ void TPrmTmplLib::load_( TConfig *icfg )
     }
 
     //Check for remove items removed from DB
-    if(!SYS->selDB().empty()) {
-	vector<string> it_ls;
-	list(it_ls);
-	for(unsigned i_it = 0; i_it < it_ls.size(); i_it++)
-	    if(itReg.find(it_ls[i_it]) == itReg.end())	del(it_ls[i_it]);
+    if(SYS->chkSelDB(SYS->selDB(),true)) {
+	vector<string> itLs;
+	list(itLs);
+	for(unsigned iIt = 0; iIt < itLs.size(); iIt++)
+	    if(itReg.find(itLs[iIt]) == itReg.end())	del(itLs[iIt]);
     }
 }
 
@@ -746,7 +878,7 @@ void TPrmTmplLib::start( bool val )
 	    isErr = true;
 	}
 
-    run_st = val;
+    runSt = val;
 
     if(isErr)	throw err_sys(_("Error starting some templates."));
 }
@@ -759,7 +891,7 @@ void TPrmTmplLib::add( const string &id, const string &name )
 TVariant TPrmTmplLib::objFuncCall( const string &iid, vector<TVariant> &prms, const string &user )
 {
     //Configuration functions call
-    TVariant cfRez = objFunc(iid, prms, user);
+    TVariant cfRez = objFunc(iid, prms, user, RWRWR_, "root:" SDAQ_ID);
     if(!cfRez.isNull()) return cfRez;
 
     return TCntrNode::objFuncCall(iid, prms, user);
@@ -770,7 +902,8 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
     //Get page info
     if(opt->name() == "info") {
 	TCntrNode::cntrCmdProc(opt);
-	ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter templates library: ")+id(),RWRWR_,"root",SDAQ_ID);
+	XMLNode *nd = ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter templates library: ")+id(),RWRWR_,"root",SDAQ_ID);
+	if(nd)	nd->setAttr("doc", TUIS::docKeyGet(descr()));
 	if(ctrMkNode("branches",opt,-1,"/br","",R_R_R_))
 	    ctrMkNode("grp",opt,-1,"/br/tmpl_",_("Template"),RWRWR_,"root",SDAQ_ID,2,"idm",OBJ_NM_SZ,"idSz",OBJ_ID_SZ);
 	if(ctrMkNode("area",opt,-1,"/lib",_("Library"))) {
@@ -807,7 +940,7 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
 	vector<string> tls;
 	list(tls);
 	time_t maxTm = 0;
-	for(unsigned i_t = 0; i_t < tls.size(); i_t++) maxTm = vmax(maxTm, at(tls[i_t]).at().timeStamp());
+	for(unsigned iT = 0; iT < tls.size(); iT++) maxTm = vmax(maxTm, at(tls[iT]).at().timeStamp());
 	opt->setText(i2s(maxTm));
     }
     else if(a_path == "/lib/cfg/ID" && ctrChkNode(opt))		opt->setText(id());
